import { helper } from '@ember/component/helper';
import { htmlSafe } from '@ember/string';
import { isNone } from '@ember/utils';
import styles from '@choiceform/ui-foundation/styles/icon';

export function icon(
  [name = null],
  {
    image = null,
    type = null,
    size = null,
    viewBox = null,
    radius = 0,
    color = null,
  }
) {
  if (isNone(size)) {
    switch (type) {
      case 'symbol':
        size = 36;
        break;
      case 'logo':
        size = 48;
        break;
      default:
        size = 16;
    }
  } else if (isNone(viewBox)) {
    switch (type) {
      case 'symbol':
        viewBox = size;
        break;
      case 'logo':
        viewBox = size;
        break;
      default:
        viewBox = size;
    }
  }

  if (isNone(viewBox)) {
    switch (type) {
      case 'symbol':
        viewBox = 36;
        break;
      case 'logo':
        viewBox = 48;
        break;
      default:
        viewBox = 16;
    }
  }

  if (name === null && image) {
    return htmlSafe(renderImageIcon(image, size, radius));
  } else {
    return htmlSafe(renderSVGIcon(name, size, viewBox, color));
  }
}

export function renderImageIcon(image, size, radius, className = 'ui-profile') {
  className = `ui-profile ${styles.image}`;
  let style = `border-radius: ${radius}px; background-image: url(${image}); width: ${size}px; height: ${size}px;`;
  return `<div class="${className}" style="${style}"></div>`;
}

export function renderSVGIcon(
  name,
  size,
  viewBox,
  color,
  className = 'ui-icon'
) {
  color = color ? `fill="${color}" ` : '';
  return (
    `<svg class="${className}" ` +
    `xmlns="http://www.w3.org/2000/svg" ` +
    `xmlns:xlink="http://www.w3.org/1999/xlink" ` +
    `x="${size}" y="${size}" width="${size}" height="${size}" ` +
    `viewBox="0 0 ${viewBox} ${viewBox}" preserveRatioAspect="xMidYMid">` +
    `<use ${color}xlink:href="#${name}"></use></svg>`
  );
}

export default helper(icon);
