'use strict';

const merge = require('ember-cli-lodash-subset').merge;

module.exports = function foundation(options) {
  let defaults = {
    // @import 模块导入规则
    import: {
      path: ['node_modules/@choiceform/ui-foundation/addon/styles'],
    },

    // 保留原生 CSS 变量（高级浏览器支持）
    preserve: false,

    // 避免和 CSS Modules 连用时注入重复的 CSS 变量
    preserveInjectedVariables: false,

    // CSS 变量
    variables: {
      /* Gray Colors */
      "--white": "#FFFFFF",
      "--black": "#000000",

      /* Primary color */
      "--primary-color": "#0057ff",
      "--primary-light-color": "var(--primary-alternatives-01)",
      "--primary-dark-color": "var(--primary-alternatives-07)",

      /* Secondary color */
      "--secondary-color": "#46cc00",
      "--secondary-light-color": "var(--secondary-alternatives-01)",
      "--secondary-dark-color": "var(--secondary-alternatives-07)",

      /* Accent color */
      "--accent-color": "#ffa800",
      "--accent-light-color": "var(--accent-alternatives-07)",
      "--accent-dark-color": "var(--accent-alternatives-01)",

      /* Triadic Colors */
      "--triadic-color": "#b11aff",
      "--triadic-dark-color": "#8600cc",
      "--triadic-light-color": "#c24dff",

      /* Split Colors */
      "--split-color": "#ff2900",
      "--split-dark-color": "#cc2100",
      "--split-light-color": "#ff694d",

      /* Bright theme palette */
      "--palette-light-00": "color-mod(var(--palette-light-05) shade(24%))",
      "--palette-light-01": "color-mod(var(--palette-light-05) shade(16%))",
      "--palette-light-02": "color-mod(var(--palette-light-05) shade(12%))",
      "--palette-light-03": "color-mod(var(--palette-light-05) shade(8%))",
      "--palette-light-04": "color-mod(var(--palette-light-05) shade(4%))",
      "--palette-light-05": "var(--white)",

      /* Dark theme palette */
      "--palette-dark-01": "var(--black)",
      "--palette-dark-02": "color-mod(var(--palette-dark-01) lightness(+4%))",
      "--palette-dark-03": "color-mod(var(--palette-dark-01) lightness(+8%))",
      "--palette-dark-04": "color-mod(var(--palette-dark-01) lightness(+12%))",
      "--palette-dark-05": "color-mod(var(--palette-dark-01) lightness(+16%))",

      /* Dark text on light backgrounds */
      /* Primary text */
      "--primary-dark-text": "var(--black)",
      /* Secondary text */
      "--secondary-dark-text": "color-mod(var(--primary-dark-text) lightness(+32%))",
      /* Disabled; hint text; and icons */
      "--disabled-dark": "color-mod(var(--palette-light-05) shade(32%))",
      /* Border */
      "--border-dark-01": "color-mod(var(--palette-dark-01) a(8%))",
      "--border-dark-02": "color-mod(var(--palette-dark-01) a(16%))",
      "--border-dark-03": "color-mod(var(--palette-dark-01) a(24%))",
      "--border-dark-04": "color-mod(var(--palette-dark-01) a(32%))",

      /* White text on dark backgrounds */
      /* Primary text */
      "--primary-light-text": "var(--white)",
      /* Secondary text */
      "--secondary-light-text": "color-mod(var(--primary-light-text) shade(32%))",
      /* Disabled; hint text; and icons */
      "--disabled-light": "color-mod(var(--palette-dark-01) lightness(+24%))",
      /* Border */
      "--border-light-01": "color-mod(var(--palette-light-05) a(16%))",
      "--border-light-02": "color-mod(var(--palette-light-05) a(24%))",
      "--border-light-03": "color-mod(var(--palette-light-05) a(32%))",

      /* Primary Fade Colors */
      "--primary-fade-00": "color-mod(var(--primary-color) a(4%))",
      "--primary-fade-01": "color-mod(var(--primary-color) a(8%))",
      "--primary-fade-02": "color-mod(var(--primary-color) a(16%))",
      "--primary-fade-03": "color-mod(var(--primary-color) a(24%))",
      "--primary-fade-04": "color-mod(var(--primary-color) a(32%))",
      "--primary-fade-05": "color-mod(var(--primary-color) a(48%))",
      "--primary-fade-06": "color-mod(var(--primary-color) a(64%))",

      /* Secondary Fade Colors */
      "--secondary-fade-00": "color-mod(var(--secondary-color) a(4%))",
      "--secondary-fade-01": "color-mod(var(--secondary-color) a(8%))",
      "--secondary-fade-02": "color-mod(var(--secondary-color) a(16%))",
      "--secondary-fade-03": "color-mod(var(--secondary-color) a(24%))",
      "--secondary-fade-04": "color-mod(var(--secondary-color) a(32%))",
      "--secondary-fade-05": "color-mod(var(--secondary-color) a(48%))",
      "--secondary-fade-06": "color-mod(var(--secondary-color) a(64%))",

      /* Accent Fade Colors */
      "--accent-fade-00": "color-mod(var(--accent-color) a(4%))",
      "--accent-fade-01": "color-mod(var(--accent-color) a(8%))",
      "--accent-fade-02": "color-mod(var(--accent-color) a(16%))",
      "--accent-fade-03": "color-mod(var(--accent-color) a(24%))",
      "--accent-fade-04": "color-mod(var(--accent-color) a(32%))",
      "--accent-fade-05": "color-mod(var(--accent-color) a(48%))",
      "--accent-fade-06": "color-mod(var(--accent-color) a(64%))",

      /* Triadic Fade Colors */
      "--triadic-fade-00": "color-mod(var(--triadic-color) a(4%))",
      "--triadic-fade-01": "color-mod(var(--triadic-color) a(8%))",
      "--triadic-fade-02": "color-mod(var(--triadic-color) a(16%))",
      "--triadic-fade-03": "color-mod(var(--triadic-color) a(24%))",
      "--triadic-fade-04": "color-mod(var(--triadic-color) a(32%))",

      /* Split Fade Colors */
      "--split-fade-00": "color-mod(var(--split-color) a(4%))",
      "--split-fade-01": "color-mod(var(--split-color) a(8%))",
      "--split-fade-02": "color-mod(var(--split-color) a(16%))",
      "--split-fade-03": "color-mod(var(--split-color) a(24%))",
      "--split-fade-04": "color-mod(var(--split-color) a(32%))",

      /* Primary Alternatives Colors */
      "--primary-alternatives-01": "#0097ff",
      "--primary-alternatives-02": "#0082ff",
      "--primary-alternatives-03": "#006cff",
      "--primary-alternatives-04": "var(--primary-color)",
      "--primary-alternatives-05": "#0042ff",
      "--primary-alternatives-06": "#002cff",
      "--primary-alternatives-07": "#0017ff",

      /* Secondary Alternatives Colors */
      "--secondary-alternatives-01": "#79cc00",
      "--secondary-alternatives-02": "#68cc00",
      "--secondary-alternatives-03": "#57cc00",
      "--secondary-alternatives-04": "var(--secondary-color)",
      "--secondary-alternatives-05": "#35cc00",
      "--secondary-alternatives-06": "#24cc00",
      "--secondary-alternatives-07": "#13cc00",

      /* Accent Alternatives Colors */
      "--accent-alternatives-01": "#ff6800",
      "--accent-alternatives-02": "#ff7e00",
      "--accent-alternatives-03": "#ff9300",
      "--accent-alternatives-04": "var(--accent-color)",
      "--accent-alternatives-05": "#ffbd00",
      "--accent-alternatives-06": "#ffd300",
      "--accent-alternatives-07": "#ffe800",

      /* Monochromacy Colors */
      "--achromatopsia": "#505050",
      "--atypical-achromatopsia": "#405173",

      /* Typography */
      "--font-size": "1rem",
      "--font-size-xx-small": "calc(var(--font-size-medium) * .5)",
      "--font-size-x-small": "calc(var(--font-size-medium) * .625)",
      "--font-size-small": "calc(var(--font-size-medium) * .75)",
      "--font-size-smaller": "calc(var(--font-size-medium) * .875)",
      "--font-size-medium": "1rem",
      "--font-size-large": "calc(var(--font-size-medium) * 1.125)",
      "--font-size-larger": "calc(var(--font-size-medium) * 1.25)",
      "--font-size-x-large": "calc(var(--font-size-medium) * 1.5)",
      "--font-size-xx-large": "calc(var(--font-size-medium) * 2)",
      "--font-lighter": "200",
      "--font-bold": "600",

      /* Border Radius */
      "--radius-small": "2px",
      "--radius-medium": "var(--cells-025)",
      "--radius-large": "var(--cells-05)",
      "--radius-big": "var(--cells-08)",

      /* Offset */
      "--offset": "2rem",
      "--cells-025": "calc(var(--cells-2) / 8)",
      "--cells-05": "calc(var(--cells-2) / 4)",
      "--cells-08": "calc(var(--cells-2) / 2.5)",
      "--cells-1": "calc(var(--cells-2) / 2)",
      "--cells-15": "calc(var(--cells-2) * .75)",
      "--cells-16": "calc(var(--cells-2) * .8)",
      "--cells-2": "2rem",
      "--cells-225": "calc(var(--cells-2) * 1.125)",
      "--cells-25": "calc(var(--cells-2) * 1.25)",
      "--cells-3": "calc(var(--cells-2) * 1.5)",
      "--cells-35": "calc(var(--cells-2) * 1.75)",
      "--cells-4": "calc(var(--cells-2) * 2)",

      /* Z-index */
      "--z-index-1": "1",
      "--z-index-2": "calc(var(--z-index-1) + 1)",
      "--z-index-3": "calc(var(--z-index-2) + 1)",
      "--z-index-4": "calc(var(--z-index-3) + 1)",
      "--z-index-5": "calc(var(--z-index-4) + 1)",
      "--z-index-nav": "100",
      "--z-index-loading": "calc(var(--z-index-nav) + 1)",
      "--z-index-modal": "1000",
      "--z-index-dropdown": "calc(var(--z-index-modal) + 1)",
      "--z-index-notification": "calc(var(--z-index-dropdown) + 1)",
      "--z-index-tooltip": "calc(var(--z-index-notification) + 1)",

      /* Shadow */
      "--block-shadow": "0 2px 6px 0 color-mod(var(--black) a(16%))",
    },

    // Sassy 高级变量
    advancedVariables: {
      unresolved: 'warn',
    },

    // 自定义 Media Queries
    media: {
      extensions: {
        '--mobile': '(width <= 768px)',
        '--over-mobile': '(width >= 768px)',
        '--into-tablet': '(width <= 1018px)',
        '--tablet': '(width >= 768px) and (width <= 1018px)',
        '--over-tablet': '(width >= 1018px)',
        '--into-desktop': '(width <= 1358px)',
        '--desktop': '(width >= 1018px) and (width <= 1358px)',
        '--over-desktop': '(width >= 1358px)',
      },
      preserve: false,
      appendExtensions: false,
    },

    // postcss-assets 选项
    assets: {
      basePath: 'public/',
      loadPaths: ['assets/images/'],
    },

    // postcss-calc 选项
    calc: {
      // 计算精度
      precision: 3,
      // 是否保留原始表达式（而不是求值）
      preserve: false,
      // 是否允许在 media queries 声明里求值
      mediaQueries: true,
      // 是否允许在选择符里求值
      selectors: false,
    },

    // postcss-short 选项
    short: {
      features: {
        border: {
          disable: false,
        },
        borderRadius: {
          disable: false,
        },
        color: {
          disable: false,
        },
        fontSize: {
          disable: false,
        },
        fontWeights: {
          disable: false,
        },
        position: {
          disable: false,
        },
        size: {
          disable: false,
        },
        spacing: {
          disable: false,
        },
      },
      skip: '*',
    },

    // postcss-clear 选项
    clear: {
      display: 'block',
    },
  };

  options = merge({}, defaults, options);

  return {
    babel: {
      plugins: [['@babel/plugin-proposal-object-rest-spread', {}]],
      sourceMaps: process.env.EMBER_ENV !== 'production' ? 'inline' : false,
    },

    // 本地模块化样式选项
    cssModules: {
      // PostCSS 选项
      postcssOptions: {
        // 使用 PostCSS 生成样式的 sourcemaps，用于浏览器内对应模块的跳转（推荐）
        map: process.env.EMBER_ENV !== 'production',
      },

      // PostCSS 插件，它们的选项在本文件的前面
      plugins: {
        before: [
          require('postcss-import')(options.import),
          require('postcss-nesting'),
          require('postcss-extend'),
          require('postcss-css-variables')({
            preserve: options.preserve,
            preserveInjectedVariables: options.preserveInjectedVariables,
            variables: options.variables,
          }),
          require('postcss-custom-media')(options.media),
          require('postcss-media-minmax'),
          require('postcss-advanced-variables')(options.advancedVariables),
          require('postcss-calc')(options.calc),
          require('postcss-assets')(options.assets),
          require('postcss-object-fit-images'),
          require('postcss-color-mod-function')({
            transformVars: true,
            unresolved: 'warn',
          }),
          require('postcss-image-set-function')({
            preserve: options.preserve,
          }),
          require('postcss-fallback'),
          require('postcss-mobile-hover'),
          require('postcss-short')(options.short),
          require('postcss-clearfix')(options.clear),
        ],
        postprocess: [
          require('postcss-merge-rules'),
          require('postcss-discard-empty'),
          require('postcss-discard-comments'),
          require('postcss-discard-duplicates'),
        ],
      },
    },

    // 是否默认包含 device.js
    useDeviceJS: false,

    // 是否默认包含 normalize.css（推荐）
    useNormalizeCSS: false,

    // 是否默认包含 object-fit 的 polyfill
    // Usage: https://github.com/bfred-it/object-fit-images#usage
    useObjectFitImagesPolyfill: false,

    // 是否默认导入 SVG 图标（'survey-client' 将导入专门为 os-client 设计的图标集），`false` 则不导入任何图标
    importSVGIcon: false,

    // 是否将应用的环境选项作为 <meta> 标签注入到 HTML 文档中，否则编译到最终生成的 js 代码中
    storeConfigInMeta: process.env.EMBER_ENV === 'test',

    // 是否在产品环境下禁用 Ember Inspector
    disableInspectorInProductionMode: true,
  };
};
